import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, ParamMap, Router } from '@angular/router';
import { Organization } from 'src/app/models/organization.model';
import { OrganizationService } from 'src/app/services/organization.service';
import { FormBuilder, FormGroup, Validators, FormArray } from '@angular/forms';
import { tap } from 'rxjs/operators';

@Component({
  selector: 'app-organization-form',
  templateUrl: './organization-form.component.html',
  styleUrls: ['./organization-form.component.scss'],
})
export class OrganizationFormComponent implements OnInit {

  organization: Organization;
  form: FormGroup;
  title: string;

  constructor(
    private organizationService: OrganizationService,
    private route: ActivatedRoute,
    private router: Router,
    private _fb: FormBuilder,
  ) {
  }

  ngOnInit() {
    this.title = this.route.snapshot.data.title;
    this.initForm();

    this.route.paramMap
      .filter((paramMap: ParamMap) => (paramMap.get('id') !== null))
      .switchMap((paramMap: ParamMap) => this.organizationService.findById(paramMap.get('id')))
      .subscribe((organization: Organization) => {

        this.organization = organization;

        const arr = new FormArray([]);
        this.organization.links.forEach(y => {
          arr.push(this._fb.group({
            url: y.url,
            organizationId: y.organizationId,
          }));
        });

        this.form = this._fb.group(
          {
            id: [this.organization.id],
            name: [organization.name, Validators.required],
            description: [organization.description, Validators.required],
            logo: [organization.logo],
            elasticSearchName: [organization.elasticSearchName],
            links: arr,
          });

      });

  }

  initForm() {
    this.organization = new Organization();
    const arr = new FormArray([]);
    this.form = this._fb.group(
      {
        id: [this.organization.id],
        name: [this.organization.name, Validators.required],
        description: [this.organization.description, Validators.required],
        logo: [this.organization.logo],
        elasticSearchName: [this.organization.elasticSearchName],
        links: arr,
      });
  }

  initItemLinkForm() {
    return this._fb.group({
      name: '',
      url: ['', Validators.required],
    });
  }

  addLink() {
    if (!this.form.controls.links) {
      this.organization.links = [];
    }
    (this.form.controls.links as FormArray).push(this._fb.group(
      { id: null, name: '',
      url: ['', Validators.required],
      organizationId: this.organization.id }));
  }

  removeLink(index) {
    (this.form.controls.links as FormArray).removeAt(index);
  }

  get formLinks() {
    return this.form.controls.links as FormArray;
  }

  onSubmit() {
    if (! this.formInvalid) {
      this.organization = new Organization(this.form.value);
      this.organizationService.replaceOrCreate(this.organization)
        .subscribe(
          (organizationCreated) => {
            this.router.navigate(['/organizations', organizationCreated.id])
              .then(() => {
              });
          },
          (err) => {
            alert(err.message);
          },
        );
    }
  }

  // Getters for each property
  get name() {
    return this.form.controls['name'];
  }

  get description() {
    return this.form.controls['description'];
  }

  get formInvalid() {
    return this.form.invalid;
  }
}
